// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  CatalogDoc.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe Documento di Catalogo

#import 	"CatalogDoc.h"

#import		"LSFormatter.h"
#import		"FileStruct.h"
#import		"UserPrefs.h"
#import		"ImageAndTextCell.h"
#import		"djZeroUtils.h"
#import		"WaitPanCtrl.h"
#import		"ChooseVolCtrl.h"

// funzioni ausiliarie alla gestione
BOOL	deleteItemFromHierarchy ( NSMutableArray * hier , FileStruct * fItem );
void	attachFormatter( NSTableColumn *outView, NSString * colId ) ;
void	setupColumn( NSOutlineView *outView, BOOL addRem, NSString * colId ) ;

@implementation CatalogDoc

// ----------------------------------------------------------------------------------------
// per distruggere il catalogo, devo buttar via i dati
- (void)
dealloc
{
	// elimino me stesso dagli osservatori
	[[NSNotificationCenter defaultCenter] removeObserver:self];
	[ dataSource release ];
	[ dataFromFile release ];
	[ super dealloc ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// per caricare la finestra dal nib, ecco il nome
- (NSString *)
windowNibName
{
	return @"CatalogWin";
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// poi, quando la finestra e' stata caricata...
- (void)
windowControllerDidLoadNib:	(NSWindowController *) aController
{
	ImageAndTextCell *imageAndTextCell = nil;
	NSTableColumn *tableColumn = nil;
	
	// invoco il metodo della super-classe
	[super windowControllerDidLoadNib:aController];
	// devo costruire l'oggetto sorgente dei dati
	[ self setDataSource: [[LSDataSource alloc] init ] ];
	// e' la sorgente di dati di outlineView
	[ outlineView setDataSource: dataSource];
	// imposto l'ordine iniziale
	[ dataSource setOrderColumn: [ NSString stringWithString: COLID_FILENAME]];
	[ dataSource setOrderDirection: TRUE ];
	// carico i dati nella finestra
	[ self loadCatalogWithData: dataFromFile] ;
	// registro la finestra che accetti drag
	[ [ aController window] registerForDraggedTypes: 
		[NSArray arrayWithObject: NSFilenamesPboardType]  ];
	// salvo nelle prefs le dimensioni delle colonne della outlineView
	[ outlineView setAutosaveName: @"outlineViewOption" ];
	[ outlineView setAutosaveTableColumns: YES ];
	[ outlineView setDelegate: self ];
	
	// attacco la toolbar alla finestra
	[self addToolbarToWin: [ aController window] ];
	
	// inserisco una cella custom nella outlineView; la cella standard gestisce solo
	// testo, questa nuova testo ed una immagine
	tableColumn = [outlineView tableColumnWithIdentifier: COLID_FILENAME];
	// creo l'oggetto autorelease perche' passa in carico alla tableColumn
	imageAndTextCell = [[[ImageAndTextCell alloc] init] autorelease];
	[imageAndTextCell setEditable: NO];
	[tableColumn setDataCell:imageAndTextCell];
	
	// assegno alla colonna presente l'immagine che indica l'ordinamento
	[ outlineView setIndicatorImage: [NSImage imageNamed: @"imgUpS"] inTableColumn: tableColumn ];
	[ outlineView setTarget: self ];
	// facendo doppio clic, si cambia la direzione di ordinamento
	[ outlineView setDoubleAction: @selector( changeSortOrder:) ];
	
	// mi abbono all'evento: pref aggiornate
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( prefsUpdated: )
		name: PREF_UPDATE_NOTIFICATION object: nil ] ;
	// recupero le pref ed adeguo la finestra
	// in realta', faccio finta che siano state aggiornate le prefs
	[[NSNotificationCenter defaultCenter]
		postNotificationName: PREF_UPDATE_NOTIFICATION object:self];
}

- (void)
loadCatalogWithData: (NSData *) data
{
	if ( data )
	{
		NSMutableArray * catData = [NSUnarchiver unarchiveObjectWithData: data ];
		// assegno il tutto alla sorgente di dati
		[ dataSource setStartPoint: catData ] ;
	}
	// dico che sono cambiate le cose
	[ outlineView reloadData ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodi utili al controllore 
- (NSData *)
dataRepresentationOfType:(NSString *)aType
{
	return ( [NSArchiver archivedDataWithRootObject: [dataSource startPoint] ] );
}

- (BOOL)
loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
	if ( outlineView )
	{
		[ self loadCatalogWithData: data ] ;
	}
	else
	{
		dataFromFile = [ data retain ];
	}
	return ( YES );
}

// ----------------------------------------------------------------------------------------
// aggiunta di un elemento al catalogo
- (void) 
addFileItem: (id)sender
{
#if	ADD_ONLY_VOLUMES
	// se devo aggiungere solamente volumi, costruisco una finestra
	// in cui  presente la lista dei volumi; sara' una sheet
	// costruisco la finestra
	ChooseVolCtrl *tmpCVCtrl = [[ ChooseVolCtrl alloc] init ] ;
	// comincio la procedura: mi metto modale per il documento
	// alla chisura della sheet, sara' invocato il metodo addVolume:...
	[ NSApp beginSheet: [ tmpCVCtrl window ]
		modalForWindow: [ NSApp mainWindow ]  
		modalDelegate: self
		didEndSelector: @selector( addVolume2Cat:returnCode:contextInfo: )  
		contextInfo: nil
	];
#else
	int risposta;
	// prendo la finestra Open generale
	NSOpenPanel *oPanel = [ NSOpenPanel openPanel ];
	// la particolarizzo per le mie esigenze
	[ oPanel setFrame: NSMakeRect(0, 0, 500, 200) display: NO ];
	[ oPanel setTitle:@"Aggiungi File" ];
	[ oPanel setPrompt:@"Seleziona un file qualsiasi" ];
	[ oPanel setCanChooseDirectories:YES ];
	[ oPanel setCanChooseFiles:YES ];
	[ oPanel setAllowsMultipleSelection:NO ];
	[ oPanel setResolvesAliases:NO ];
	// faccio la richiesta all'utente
	risposta = [oPanel runModalForTypes: nil];
	// quando arrivo qui, l'utente ha risposto
	if (risposta == NSOKButton) 
	{
		// in particolare, ha scelto qualcosa
		// recupero la lista dei file prescelti
		NSArray *filesToOpen = [ oPanel filenames ];
		// ce ne e' sempre uno solo... lo piglio
		NSString *aFile = [ filesToOpen objectAtIndex: 0 ];
		// costruisco l'alberatura dei file a partire dalla scelta
		FileStruct * fInfo = [[ FileStruct alloc ] initTreeFromPath: aFile ];
		// aggiungo la cosa al catalogo
		[ dataSource addFileEntry: fInfo ];
		// ovviamente, il documento e' stato sporcato
		[ self updateChangeCount: NSChangeDone ] ;
		// rinfresco la finestra con i nuovi dati
		[ outlineView reloadData ];
	}
#endif
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

#if	ADD_ONLY_VOLUMES

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// al termine delle operazioni svolte dalla sheet, e' invocato questo metodo
// con returnCode, scelgo cosa fare
// -1 significa fare nulla,
// altrimenti, e' l'indice del volume da catalogare
- (void)
addVolume2Cat:		(NSWindow *)sheet 
	returnCode:	(int)returnCode 
	contextInfo:	(void  *)contextInfo
{
	NSString	* volName ;

	// se il codice di ritorno e' -1, non ho nulla da fare
	if ( returnCode== -1 )
		return ;
	// se arrivo qui, ho da montare un volume
	// recupero il percorso completo del volume...
	// dalla finestra recupero il il controllore, da qui il vettore dei percorsi, e dal vettore
	// piglio l'elemento di indice returnCode
	volName = [[((ChooseVolCtrl*) [sheet windowController]) volumePaths] objectAtIndex: returnCode ];
	[ self performAddFilesModal: [ NSArray arrayWithObject: volName]];
	// ovviamente, il documento e' stato sporcato
	[ self updateChangeCount: NSChangeDone ] ;
	// rinfresco la finestra con i nuovi dati
	[ outlineView reloadData ];
}
#endif

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// aggiunge al catalogo gli elementi nello array
- (void)
performAddFilesModal: ( NSArray *)volList 
{
	NSModalSession 	session ;
	NSEnumerator 	* enumerator = [volList objectEnumerator];
	NSString	* volPath ;

	volPath = [enumerator nextObject] ;
	// mostro la finestra di attesa con apposita stringa
	[ WaitPanCtrl showHide: TRUE ];
	[ WaitPanCtrl setText: [NSString stringWithFormat: @"Cataloging %@", 
		[[ NSFileManager defaultManager] displayNameAtPath: volPath] ]];
	// faccio partire l'animazione della barra del barbiere
	[ WaitPanCtrl animate: self ];
	// comincio una sessione modale per l'intera applicazione
	session = [NSApp beginModalSessionForWindow: [ [ WaitPanCtrl sharedProgress] window ] ];
	// eseguo la sessione
	[NSApp runModalSession:session] ;
	while ( volPath ) 
	{
		// costruisco l'alberatura dei file a partire dalla scelta
		FileStruct 	* fInfo = [[ FileStruct alloc ] initTreeFromPath: volPath ];
		// aggiungo la cosa al catalogo
		[ dataSource addFileEntry: fInfo ];
		volPath = [enumerator nextObject] ;
		if ( volPath == nil ) break ;
		// mostro la finestra di attesa con apposita stringa
		[ WaitPanCtrl setText: [NSString stringWithFormat: @"Cataloging %@", 
			[[ NSFileManager defaultManager] displayNameAtPath: volPath] ]];
		// faccio partire l'animazione della barra del barbiere
		[ WaitPanCtrl animate: self ];
	}
	// finisco la sessione
	[NSApp endModalSession: session];
	// nascondo la finestra di attesa
	[ WaitPanCtrl showHide: FALSE ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// eliminazione di un elemento dal catalogo
- (void) 
delItem: (id)sender
{
	FileStruct * fInfo ;
	// guardo se c'e' una riga selezionata
	int	rowsel = [ outlineView selectedRow ];
	if ( rowsel == -1 ) 
	{
		// no, lascio stare
		NSBeep();	// bip!!
		return ;
	}
	// se arrivo qui, c'e' una riga selezionata
	fInfo = [ outlineView itemAtRow: rowsel ] ;
	// adesso voglio eliminare questo elemento da dataSource
	// e' un vero pasticcio: devo cercare la directory di cui
	// e' figlio, cosa non facile...
	// ed allora, considero ricorsivamente gli start points e vado
	deleteItemFromHierarchy( [dataSource startPoint], fInfo );	
	// ovviamente, il documento e' stato sporcato
	[ self updateChangeCount: NSChangeDone ] ;
	[ outlineView reloadData ];
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// ----------------------------------------------------------------------------------------
// metodi accessor
- (NSData *) dataFromFile			{ return dataFromFile ; }
- (void) setDataFromFile: ( NSData*) data	OBJ_ACC_SET( dataFromFile, data )
	
- (LSDataSource *)  dataSource			{ return dataSource ; }
- (void) setDataSource: (LSDataSource*) ds	OBJ_ACC_SET( dataSource, ds )

- (NSOutlineView * ) outlineView 		{ return outlineView ; }
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// ----------------------------------------------------------------------------------------
// metodi per la gestione del drag'n'drop

- (unsigned int)
draggingEntered:(id<NSDraggingInfo>)sender
{
	NSPasteboard *pboard;

	// dico che accetto qualsiasi cosa che sia un path
	pboard = [sender draggingPasteboard];
	if ([[pboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound) {
		return NSDragOperationLink;
	}
	return NSDragOperationNone;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (unsigned int)
draggingUpdated:(id<NSDraggingInfo>)sender
{
	return NSDragOperationLink;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
draggingExited:(id<NSDraggingInfo>)sender
{
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
prepareForDragOperation:(id<NSDraggingInfo>)sender
{
	// questo e' il momento di decidere se fare il drop
#if	ADD_ONLY_VOLUMES
	NSPasteboard *pboard;

	pboard = [sender draggingPasteboard];
	// in primo luogo, droppo solo path di file
	if ([[pboard types] indexOfObject:NSFilenamesPboardType] != NSNotFound) 
	{
		// recupero la lista degli elementi draggati
		NSArray		* pList = [ pboard propertyListForType: NSFilenamesPboardType ] ;
		// la metto in un enumerator
		NSEnumerator 	* enumerator = [pList objectEnumerator];
		// recupero anche la lista dei volumi correntemente mondati
		NSArray		* volumes = [ [NSWorkspace sharedWorkspace] mountedLocalVolumePaths ] ;
		NSString	* volPath ;
		// dico che accetto il drop solo di volumi
		while ((volPath = [enumerator nextObject])) 
		{
			// se cerco di droppare un elemento che non e' un volume
			if ( ! [ volumes containsObject: volPath ] )
				return ( NO );	// il drop non si fa
		}
		// se arrivo qui, tutti gli elementi droppati sono volumi,
		// posso eseguire l'operazione
		return ( YES );
	}
	// se arrivo qui, gli elementi droppati non sono nemmeno path...
	return ( NO );	
#else
	return ( YES );
#endif
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (BOOL)
performDragOperation:(id<NSDraggingInfo>)sender
{
	NSPasteboard 	* dragPasteboard;
	NSArray		* pList ;

	// recupero la pasteboard
	dragPasteboard = [sender draggingPasteboard];
	// da qui, la lista degli elementi draggati
	pList = [ dragPasteboard propertyListForType: NSFilenamesPboardType ] ;
	[ self performAddFilesModal: pList ];
	// ovviamente, il documento e' stato sporcato
	[ self updateChangeCount: NSChangeDone ] ;
	return ( YES) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
concludeDragOperation:(id<NSDraggingInfo>)sender
{
	[ outlineView reloadData ];
	return ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// ----------------------------------------------------------------------------------------
// ricostruzione della outlineView in seguito ad una modifica delle preferenze
- (void)
prefsUpdated: ( NSNotification *) notification
{
	// recupero le preferences e mostro o meno la colonna
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColFullPath] boolValue], COLID_FULLPATH );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColModDate] boolValue], COLID_MODDATE );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColFileSize] boolValue], COLID_FILESIZE );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColGroupName] boolValue], COLID_GROUPNAME );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColOwnerName] boolValue], COLID_OWNERNAME );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColPosixPerm] boolValue], COLID_POSIXPERM );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColFileType] boolValue], COLID_FILETYPE );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColCreator] boolValue], COLID_OSCREATOR );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColOSType] boolValue], COLID_OSTYPE );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColFsFileNum] boolValue], COLID_FSFILENUM );
	setupColumn( outlineView, [[ UserPrefs getPrefValue: keyColFsNum] boolValue], COLID_FSNUM );
	[ outlineView reloadData ];
	[ outlineView sizeLastColumnToFit ] ;
}

// ----------------------------------------------------------------------------------------

#define		CatDocToolbarId		@"tbName_CatDoc"
#define		TBItId_AddItem		@"tbItem_AddItem"
#define		TBItId_DelItem		@"tbItem_DelItem"
#define		TBItId_SaveItem		@"tbItem_SaveItem"

// metodo per la costruzione della toolbar
- (void) 
addToolbarToWin: (NSWindow *) currWin
{
	// costruisco la nuova istanza della toolbar
	// la faccio autorelease perche' sara' poi ritenuta dalla finestra
	NSToolbar *toolbar = [[[NSToolbar alloc] initWithIdentifier: CatDocToolbarId] autorelease];

	// imposto le proprieta' di base della toolbar
	// permetto che l'utente la modifichi
	[toolbar setAllowsUserCustomization: YES];
	// lascio che conservi lo stato tra successivi lanci
	[toolbar setAutosavesConfiguration: YES];
	// dico che mostri solo le icone
	[toolbar setDisplayMode: NSToolbarDisplayModeIconOnly];
	// dico che il delegato della toolbar e' il documento stesso
	[toolbar setDelegate: self];
	// attacco la toolbar alla finestra
	[ currWin setToolbar: toolbar ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// dato un identificatore di item, restuisce l'item vero e proprio
	// questo metodo serve per dare corpo agli item non standard
- (NSToolbarItem *) 
toolbar: (NSToolbar *)toolbar 
	itemForItemIdentifier: (NSString *) itemIdent 
	willBeInsertedIntoToolbar:(BOOL) willBeInserted 
{
	// costruisco una istanza di item della toolbar
	// come al solito, e' autorelase, verra' ritenuto dalla toolbar
	NSToolbarItem *toolbarItem = [[[NSToolbarItem alloc] initWithItemIdentifier: itemIdent] autorelease];
	
	// item della toolbar per aggiungere un volume
	if ([itemIdent isEqual: TBItId_AddItem]) 
	{
		// do due nomi all'icona di aggiunta volume: nome per la toolbar
		[toolbarItem setLabel: @"Add"];
		// nome per il dialogo di personalizzazione
		[toolbarItem setPaletteLabel: @"Add Volume"]; 
		// predispongo la scritta di aiuto (il tooltip)
		[toolbarItem setToolTip: @"Add a Volume to the Catalog"];
		// predispongo l'icona di questo item
		[toolbarItem setImage: [NSImage imageNamed: @"Add"]];
		// quando l'item e' cliccato, deve mandare al documento (self)... 
		[toolbarItem setTarget: self];
		// ...il messaggio opportuno
		[toolbarItem setAction: @selector(addFileItem:)];
		return toolbarItem;
	}
	// ripeto il giochetto per la cancellazione di un elemento di catalogo
	if ([itemIdent isEqual: TBItId_DelItem]) 
	{
		[toolbarItem setLabel: @"Delete"];
		[toolbarItem setPaletteLabel: @"Delete Item"];
		[toolbarItem setToolTip: @"Remove an Item from the Catalog"];
		[toolbarItem setImage: [NSImage imageNamed: @"del"]];
		[toolbarItem setTarget: self];
		[toolbarItem setAction: @selector(delItem:)];
		return toolbarItem;
	}
	// e per il salvataggio del catalogo
	if ([itemIdent isEqual: TBItId_SaveItem]) 
	{
		[toolbarItem setLabel: @"Save"];
		[toolbarItem setPaletteLabel: @"Save Item"];
		[toolbarItem setToolTip: @"Save the Catalog"];
		[toolbarItem setImage: [NSImage imageNamed: @"save"]];
		[toolbarItem setTarget: self];
		// chiamo un metodo di default
		[toolbarItem setAction: @selector(saveDocument:)];
		return toolbarItem;
	}
	// gli altri dovrebbero essere standard
	return ( nil );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// questo metodo dice quali sono gli item di default della toolbar
	// sono gli item presenti alla creazione e quando l'utente ripristina
- (NSArray *) 
toolbarDefaultItemIdentifiers: (NSToolbar *) toolbar 
{
	// la lista e' l'elenco ordinato degli identificatori degli elementi
	return [NSArray arrayWithObjects:	
		TBItId_AddItem, 			// aggiunta elemento
		TBItId_SaveItem, 			// salvataggio catalogo
		NSToolbarSeparatorItemIdentifier, 	// separatore
		TBItId_DelItem,				// cancellazione
		NSToolbarSeparatorItemIdentifier, 	// separatore
		NSToolbarPrintItemIdentifier, 		// stampa catalogo
		NSToolbarFlexibleSpaceItemIdentifier, 	// spazio flessibile
		NSToolbarCustomizeToolbarItemIdentifier,// personalizzazione
		nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo delegato obbligatorio
// questo metodo dice quali sono gli item possibili della toolbar
	// sono TUTTI gli item che possono entrare nella toolbar, e quindi
	// mostrati nel dialogo di personalizzazione
- (NSArray *) 
toolbarAllowedItemIdentifiers: (NSToolbar *) toolbar 
{
	// normalmente, nessun elemento e' previsto presente, quindi bisogna
	// inserire tutti gli elementi che si pensa possano essere presenti;
	// in particolare, separatori e spazi vari...
	return [NSArray arrayWithObjects: 
		TBItId_AddItem, TBItId_SaveItem, TBItId_DelItem,	// i miei tre
		// e poi tutti gli altri ragionevoli...
		NSToolbarPrintItemIdentifier, NSToolbarShowFontsItemIdentifier,
		NSToolbarCustomizeToolbarItemIdentifier, NSToolbarFlexibleSpaceItemIdentifier, 
		NSToolbarSpaceItemIdentifier, NSToolbarSeparatorItemIdentifier, nil];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
	// il pulsante di stampa non funziona; dopotutto, nessuno gli ha detto di stampare
// metodo delegato opzionale
// questo metodo e' chiamato quando si aggiunge un elemento alla toolbar
	// questo metodo e' il posto dove inserire codice per predisporre gli item
	// notif e' la toolbar, l'item lo ricavo usando la chiave @"item"
- (void) 
toolbarWillAddItem: (NSNotification *) notif 
{
	// ricavo l'item che mi interessa
	NSToolbarItem *addedItem = [[notif userInfo] objectForKey: @"item"];
	// se l'elemento e' l'icona per stampare...
	if ([[addedItem itemIdentifier] isEqual: NSToolbarPrintItemIdentifier]) 
	{
		// aggiungo il tooltip
		[addedItem setToolTip: @"Print the Catalog"];
		[addedItem setAction: @selector(print:)];
	}
}  
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo inviato al target per validare l'item
// questo metodo serve per abilitare/disabilitare item sulla toolbar
- (BOOL) 
validateToolbarItem: (NSToolbarItem *) toolbarItem 
{
	// l'item salva va bene solo se il documento e' stato cambiato
	if ([[toolbarItem itemIdentifier] isEqual: TBItId_SaveItem]) 
		return ( [self isDocumentEdited] ) ;
	// tutti gli altri sono abilitati
	return ( YES );
}

// ----------------------------------------------------------------------------------------
// nessun item della outlineView e' editabile

- (BOOL)
outlineView:			(NSOutlineView *) outlineView 
	shouldEditTableColumn:	(NSTableColumn *) tableColumn 
	item:			(id) item 
{
	return NO;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo invocato da un clic sulla colonna

- (BOOL)
outlineView:(NSOutlineView *) oView 
	shouldSelectTableColumn:(NSTableColumn *)tableColumn
{
	NSArray		* tabLists = [ oView tableColumns ];
	NSEnumerator 	* enumerator = [tabLists objectEnumerator];
	NSTableColumn	* tc ;

	// in mancanza di metodi migliori, tolgo a tutte le colonne 
	// l'indicatore di ordinamento (ho gia' ricavato un enumerator
	// con tutte le colonne al momento presenti)
	while ((tc = [enumerator nextObject])) 
		[ oView setIndicatorImage: nil inTableColumn: tc ];
	// imposto l'immagine nello header della colonna
	if ( [ [ self dataSource] orderDirection ])
		[ oView setIndicatorImage: [NSImage imageNamed: @"imgUpS"] inTableColumn: tableColumn ];
	else	[ oView setIndicatorImage: [NSImage imageNamed: @"imgDwS"] inTableColumn: tableColumn ];
	// assegno la nuova chiave di ordinamento al dataSource
	[ [ self dataSource] setOrderColumn: [ tableColumn identifier]];
	// rinfresco la finestra con i nuovi dati
	[ oView reloadData ];
	return YES;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo invocato da un doppio clic su di un elemento della outlineView
// mi serve effettivametne solo per un doppio clic sullo header di una colonna

- (void)
changeSortOrder: (id) sender 
{
	NSTableColumn	* tableColumn ;

	// voglio che sia selezionata una intera colonna
	if ( [ sender selectedColumn ] == -1 ) return ;
	// determino allora quale colonna e' selezionata
	tableColumn = [ [outlineView tableColumns] objectAtIndex: [ outlineView clickedColumn ]];
	// rovescio l'ordine negando il valore corrente
	[ [ self dataSource] setOrderDirection: ! [ [ self dataSource] orderDirection ] ];
	// aggiusto l'immagine nello header della colonna
	if ( [ [ self dataSource] orderDirection ])
		[ sender setIndicatorImage: [NSImage imageNamed: @"imgUpS"] inTableColumn: tableColumn ];
	else	[ sender setIndicatorImage: [NSImage imageNamed: @"imgDwS"] inTableColumn: tableColumn ];
	// rinfresco la finestra con i nuovi dati
	[ outlineView reloadData ];
}

// ----------------------------------------------------------------------------------------
- (void)
changeFont: (id) sender
{
	NSFont 		*oldFont , *newFont ;
	NSTableColumn	* locTC ;
	short		selItem ;
	NSTextFieldCell		*dc ;

	selItem = [ outlineView selectedColumn ] ;
	if ( selItem == -1 )
	{
		NSBeep( ) ;
		return ;
	}
	// se arrivo qui, colonna selezionata
	// recupero in qualche modo la TableColumn corrispondente
	locTC = [[ outlineView tableColumns] objectAtIndex: selItem ];
	dc = [ locTC dataCell ] ;
	oldFont = [dc font];
	newFont = [sender convertFont:oldFont];	
	[ dc setFont: newFont ];
	[locTC setDataCell: dc ];
	[ outlineView reloadData ];
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
@end

// ----------------------------------------------------------------------------------------
// cancella un elemento dalla lista file esplorando ricorsivamente l'albero
BOOL
deleteItemFromHierarchy ( NSMutableArray * hier , FileStruct * fItem )
{
	int 		numOfItem, i;

	// se lo item e' nell'array, bene
	if ([  hier containsObject: fItem ])
	{
		// lo rimuovo direttamente e dico che ho finito
		[ hier removeObject: fItem ] ;
		return ( TRUE );
	}
	// non c'e', lo cerco in tutti i suoi figli
	numOfItem = [ hier count ] ;	// conto i figli e ciclo
	for ( i = 0 ; i < numOfItem ; i ++ )
	{
		// questo e' l'elemento corrente
		FileStruct * locRoot = [ hier objectAtIndex: i ];
		// cerco ricorsivamente l'elemento da cancellare
		if ( deleteItemFromHierarchy ( [locRoot fileList] , fItem ) )
			return ( TRUE );
	}
	// se arrivo qui (ma non dovrebbe succedere), non ho ancora trovato il file
	return ( FALSE );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// modifica una tablecolumn aggiungendo formatter
void
attachFormatter( NSTableColumn *tc, NSString * colId )
{
	// i formatter sono creati autorelease perche' passano in gestione alla tablecolumn
	if ( [ colId isEqual: COLID_OSTYPE ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_OSCREATOR ] )
	{
		TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF1 ];
		return ;
	}
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		FileSizeForm 	*myDF2 = [[[ FileSizeForm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF2 ];
		// gia' che ci sono, sbandiero a destra
		[[tc dataCell   ] setAlignment: NSRightTextAlignment];
		[[tc headerCell ] setAlignment: NSRightTextAlignment];
		return ;
	}
	if ( [ colId isEqual: COLID_POSIXPERM ] )
	{
		FilePosixPerm 	*myDF3 = [[[ FilePosixPerm alloc ] init ] autorelease ];
		[[tc dataCell ] setFormatter:  myDF3 ];
		return ;
	}
	if ( [ colId isEqual: COLID_MODDATE ] )
	{
		NSDateFormatter *myDF4 = [[NSDateFormatter alloc]
			initWithDateFormat:@"%d %b %Y" allowNaturalLanguage:YES];
		[[tc dataCell ] setFormatter:  myDF4 ];
		return ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// aggiunge o elimina una colonna di una tablecolumn
// addRem = TRUE, aggiunge, FALSE elimina
void
setupColumn( NSOutlineView *outView, BOOL addRem, NSString * colId )
{
	NSTableColumn	*tmpTC ;
	// vedo se esiste una colonna relativa
	tmpTC = [ outView tableColumnWithIdentifier: colId] ;
	// se non c'e', e devo rimuoverla...
	if ( (tmpTC == nil) && (addRem == FALSE) )
		return ; 	// devo fare nulla
	// se c'e', e devo aggiungerla
	if ( tmpTC != nil && addRem )
		return ;	// devo fare nulla
	// se non c'e', e devo aggiungerla
	if ( tmpTC == nil && addRem )
	{
		// creo la NSTableColumn con l'identificatore
		tmpTC = [ [NSTableColumn alloc] initWithIdentifier: colId ];
		// imposto il titolo della colonna
		[[tmpTC headerCell] setStringValue: NSLocalizedString( colId, colId ) ];
		// attacco eventualmente un formattatore
		attachFormatter( tmpTC, colId );
		// aggiungo la colonna
		[outView addTableColumn: tmpTC ];
		return ;	// ho finito
	}
	// se arrivo qui, sicuramente c'e' la colonna e devo rimuoverla
	[outView removeTableColumn: tmpTC ];
}
