// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  FileStruct.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe che modella le informazioni gerachiche

#import "FileStruct.h"

#import		"djZeroUtils.h"
#import		"WaitPanCtrl.h"
#import		"FindWinCtrl.h"

// funzione per la ricerca con determinate opzioni
BOOL	checkSearch( LSFileInfo * item, id searchItem, NSString * colId, int opt );

@implementation FileStruct

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// inizializzazione dell'istanza in base al nome completo
- (id) initTreeFromPath: (NSString*) fullPath 
{
	// mi metto via un file manager 
	NSFileManager 	*fileManager = [NSFileManager defaultManager];
	BOOL		isAdir, fileOK ;
	int		i ;

	// per prima cosa, inizializzo super
	[ super initWithPath: fullPath ] ;
	// e adesso, se il file puntato e' una directory, espando 	
	fileOK = [ fileManager fileExistsAtPath: fullPath isDirectory: & isAdir ];
	// evito tuttavia di espandere link simbolici (che NON sono alias...)
	if ( [ [self fileType] isEqual: NSFileTypeSymbolicLink ])
		isAdir = FALSE ;
	[ self setOrderColumn: COLID_FILENAME ];
	orderDirection = TRUE ;
	// espando sempre, purche' il file esista e sia una directory effettiva
        if ( fileOK && isAdir) 
	{
		// variabile per tenere traccia della dimensione parziale
		long long	dirSize = 0;
		// vettore destinato a contenere i file
		NSMutableArray 	*tmpfileList  ;
		// l'elenco dei file 
		NSArray *dirContent = [ fileManager directoryContentsAtPath: fullPath ];
		// conto quanti file ci sono all'interno 
		int numFile = [ dirContent count ];
		// costruisco un vettore di dimensione adatta
		tmpfileList = [[ NSMutableArray alloc ] initWithCapacity: numFile ] ;
		// adesso, per ogni elemento, costruisco l'albero e poi lo aggiungo
		for ( i = 0; i < numFile; i++) 
		{
			FileStruct	* tmpFile ;
			NSString 	* myfile = [ dirContent objectAtIndex: i ] ;

			// costruisco l'albero
			tmpFile = [[FileStruct alloc]  initTreeFromPath:
				[ fullPath stringByAppendingPathComponent: myfile]] ;
			// aggiungo la dimensione (cumulata) alla directory corrente
			dirSize += [ tmpFile fileSize ];
			// aggiungo il file alla lista
			[tmpfileList addObject: tmpFile ];

			[ WaitPanCtrl setText: [NSString stringWithFormat: @"Cataloging %@", myfile ]];

		}
		// assegna al file la nuova dimensione
		[ self setFileSize: dirSize ];
		// adesso copio il vettore con la giusta dimensione
		fileList = [[ NSMutableArray alloc ] initWithCapacity: [ tmpfileList count] ];
		[ fileList setArray: tmpfileList ];
		[ self sortFileList: COLID_FILENAME direction: TRUE ];
        } 
	else 
	{
		// va beh, e' un file normale
		fileList = nil ;
        }
	return ( self );
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void) dealloc
{
	// devo distruggere solo il vettore con i file
	[ fileList release ] ;
	[ orderColumn release ];
	[ super dealloc ];
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)encodeWithCoder:(NSCoder *)encoder 
{
	int dim ;
	// salvo le variabili ereditate
	[ super encodeWithCoder: encoder ];
	// vedo quanti file ci sono
	dim = [ self numOfFiles] ;
	// metto da parte questo numero
	[ encoder encodeValueOfObjCType: @encode(int) at: & dim];
	// se ci sono file dipendenti, li salvo tutti
	if ( dim > 0 )		
		[ encoder encodeObject: [ self fileList ]];
}

- (id)initWithCoder:(NSCoder *)decoder 
{
	int dim ;
	// recupero le variabili ereditate
	[ super initWithCoder: decoder ];
	// vedo se ci sono file dipendenti
	[ decoder decodeValueOfObjCType: @encode(int) at: & dim];
	if ( dim <= 0 )
		[ self setFileList: nil ]; // non ce ne sono
		// recupero le info di tutti i file contenuti
	else 	[ self setFileList: [ decoder decodeObject ]];	
	return self ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo di comodo per recuperare un file al posto i-simo
- (FileStruct *)getFileAtIndex:(int) elem 
{
    return [[ self fileList ] objectAtIndex: elem ];
}

// metodo di comodo per vedere quanti file sono presenti nella directory
- (int)numOfFiles 
{
	return ( [[self fileList] count] ) ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

- (void) 
sortFileList: (NSString*) type 
	direction: (BOOL) dir 
{
	NSArray * newArray ;
	NSMutableArray * tmpArray ;

	// se non ci sono elementi da ordinare, abbiamo finito
	if ( ! [self fileList]  ) return ;
	// il vettore e' gia' ordinato se chiave e direzione coincidono
	if ( [ orderColumn isEqual: type ] && dir == orderDirection )
		return ;
	// altrimenti, bisogna ordinare il vettore; uso una delle due funzioni
	// a seconda della direzione di ordinamento
	if ( dir == TRUE )
		newArray = [ [self fileList] sortedArrayUsingFunction: ascSortFunc context: type] ;
	else	newArray = [ [self fileList] sortedArrayUsingFunction: desSortFunc context: type] ;
	// trasformo il risultato in un NSMutableArray
	tmpArray = [ NSMutableArray arrayWithCapacity: [ newArray count]];
	[ tmpArray addObjectsFromArray: newArray ];
	// assegno le nuove variabili d'istanza
	[ self setFileList: tmpArray];
	[ self setOrderColumn: type ];
	orderDirection = dir ;
	return ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// classici metodi accessor
- (NSMutableArray*)fileList				{ return  fileList; }
- (void)setFileList:(NSMutableArray*)newFileList	OBJ_ACC_SET( fileList, newFileList )

- (NSString*)orderColumn			{ return orderColumn; }
- (void)setOrderColumn:(NSString*)new		OBJ_ACC_SET( orderColumn, new )


// --------------------------------------------------------------------- 
// metodo che effettua una ricerca di tutti gli elementi che rispondono
// ad un dato criterio, e li infila in un vettore

- (void) 
searchItemFor: (id) item2search 
	column: (NSString*) colId 
	options: (int) opt 
	placeIn: (NSMutableArray*) foundItems 
{
	// piglio un elenco di tutti gli elementi presenti
	NSEnumerator 	* enumerator = [[self fileList] objectEnumerator];
	FileStruct	* item ;
	// prima verifico se l'elemento corrente mi va bene
	if ( checkSearch( self, item2search, colId, opt) )
		[ foundItems addObject: self ] ;
	// poi esamino tutti i file contenuti
	while (item = [enumerator nextObject])
	{
		[ item searchItemFor:  item2search column: colId 
			options: opt  placeIn:  foundItems ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// espande tutti gli elementi fino ad un elemento obiettivo

- (void) 
expandFStoItem: (NSEnumerator*) pathComp inOutlineView: (NSOutlineView *) outView
{
	// piglio l'elenco dei file contenuti
	NSEnumerator 	* enumerator = [[self fileList] objectEnumerator];
	// questo e' il (pezzo di) nome dell'elemento obiettivo
	NSString	* name = [ pathComp nextObject ];
	FileStruct	* item ;

	// esamino tutti gli elementi presenti
	while (item = [enumerator nextObject])
	{
		// se il nome coincide con quello cercato
		if ( [[item fileName] isEqual:  name] )
		{
			int	thisRow = [outView rowForItem: item];
			// dico che questo item va espanso perche' fa parte della
			// catena che porta all'obiettivo
			[ outView expandItem: item ];
			// lo seleziono
			[ outView selectRow: thisRow  byExtendingSelection: FALSE ];
			// e lo porto in bella vista
			[ outView scrollRowToVisible: thisRow ];
			// poi proseguo nell'espansione
			[ item expandFStoItem: pathComp inOutlineView: outView] ;
			// non devo esplorare gli altri elementi...
			return ;
		}
	}
}

@end

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// determina se un elemento risponde al criterio di ricerca

BOOL
checkSearch( LSFileInfo * item, id searchItem, NSString * colId, int opt )
{
	// colonne che contengono stringhe
	NSArray *id_string = [NSArray arrayWithObjects:
		COLID_FILENAME, COLID_FULLPATH, COLID_GROUPNAME,
		COLID_OWNERNAME, COLID_FILETYPE, nil ];
	// colonne che contengono numeri
 	NSArray *id_number = [NSArray arrayWithObjects:
		COLID_FILESIZE, COLID_POSIXPERM, COLID_OSCREATOR,
		COLID_OSTYPE, COLID_FSFILENUM, COLID_FSNUM, nil ];

	// tratto a parte le date
	if ( [ colId isEqual: COLID_MODDATE ] )
	{
		// confronto tra data
		return ( [ [ item modDate] isEqualToDate: searchItem]);
	}
	// confronto sulla dimensione del file (sempre per il solito misterioso
	// motivo i long long non funzionano con valueForKey )
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		// ricavo le due dimensioni
		long long	tmp1 = [ item fileSize ] ;
		long long	tmp2 = (long long) [ searchItem doubleValue ] ;
		// e le confronto brutalmente secondo l'opzione
		switch ( opt ) {
		case MENUTAG_NUM_GREAT :
			return ( tmp1 > tmp2 );
		case MENUTAG_NUM_GREATEQ :
			return ( tmp1 >= tmp2 );
		case MENUTAG_NUM_EQ :
			return ( tmp1 == tmp2 );
		case MENUTAG_NUM_LESSEQ : 
			return ( tmp1 <= tmp2 );
		case MENUTAG_NUM_LESS :
			return ( tmp1 < tmp2 );
		default :
			return ( FALSE );
		}
	}
	// poi, per tutte le colonne che sono rappresentate da stringa
	if ( [ id_string containsObject: colId ] )
	{
		// ricavo la striga 
		NSString *s1 = [ item valueForKey: colId ];
		// a seconda delle opzioni, faccio adeguato confronto
		if ( opt == MENUTAG_STR_ISEQUAL )
		{
			// confronto esatto tra stringhe
			return ( [ s1 isEqualToString: searchItem ]);
		}
		else if ( opt == MENUTAG_STR_ISCONTIANED )
		{
			NSRange	xx = [ s1 rangeOfString: searchItem ];
			return ( xx.location != NSNotFound ) ;
		}
		return ( FALSE );
	}
	// poi, per tutti gli altri numeri
	if ( [ id_number containsObject: colId ] )
	{
		// ricavo il numero
		NSNumber	* n1 = [ item valueForKey: colId ];
		NSComparisonResult	xx ;
		// a seconda delle opzioni, faccio il confronto
		switch ( opt ) {
		case MENUTAG_NUM_GREAT :
		case MENUTAG_NUM_GREATEQ :
			xx = [ n1 compare: searchItem ];
			return ( xx == NSOrderedDescending ) ;
		case MENUTAG_NUM_EQ :
			// confronto esatto
			return ( [ n1 isEqualToNumber: searchItem ]);
		case MENUTAG_NUM_LESSEQ : 
		case MENUTAG_NUM_LESS :
			xx = [ n1 compare: searchItem ];
			return ( xx == NSOrderedAscending ) ;
		default :
			return ( FALSE );
		}
	}
	return FALSE ;
}
