// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  LSDataSource.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe sorgente di dati con un catalogo di file

#import 	"LSDataSource.h"

#import		"UserPrefs.h"
#import		"ImageAndTextCell.h"
#import		"djZeroUtils.h"

// alcune funzioni ausiliarie
// funzione che discrimina quale file saltare
BOOL		skipDotFile ( NSString * fileName ) ;
// funzione che calcola il numero di file normali
short		countNormalFiles( FileStruct * directory );
// funzione che determina se una directory e' un bundle
BOOL 		checkIfBundleDirectory( NSString * fileName );
// funzione che restituisce l'i-esimo file normale di una directory
FileStruct *	getNormalFile ( FileStruct * directory, short index ) ;
int		mySortFunc( id el1, id el2, void * ctx ) ;
void		reverseArray ( NSMutableArray * locArr ) ;

@implementation LSDataSource
// --------------------------------------------------------------------- 
// inizializzazione e deallocazione: devo solo trattare il vettore
- (id ) init
{
	self = [ super init ] ;
	[self setStartPoint: [ NSMutableArray array ]];	
	[ self setOrderColumn: [ NSString stringWithString: COLID_FILENAME]];
	[ self setOrderDirection: TRUE ];
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void) dealloc 
{
	[ startPoint release ] ;	
	[ orderColumn release ] ;	
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// devo solo aggiungere un oggetto al vettore
- (void) addFileEntry: (id) newEntry 
{
	[ startPoint addObject: newEntry ];
}
// --------------------------------------------------------------------- 

// --------------------------------------------------------------------- 
// metodi accessor standard
- (NSMutableArray*)startPoint			{ return startPoint; }
- (void)setStartPoint:(NSMutableArray*)new	OBJ_ACC_SET( startPoint, new )
- (NSString*)orderColumn			{ return orderColumn; }
- (void)setOrderColumn:(NSString*)new		OBJ_ACC_SET( orderColumn, new )
- (BOOL)orderDirection				{ return orderDirection; }
- (void)setOrderDirection:(BOOL)new		{ orderDirection = new ; }

// --------------------------------------------------------------------- 

// --------------------------------------------------------------------- 
// metodi per la gestione dei dati forniti alla NSOutline
// abbiamo due problemi: i dotFiles e i bundle; i dotFiles sono i file il cui nome
// comincia con '.' (ed eventualmente i file "Icons"), che non devono essere
// visualizzati (o meglio, c'e' una pref che dice se visualizzarli o meno)
// i bundle sono le directory speciali (app, frameworks) che possono o meno
// (dipende da una pref) essere visualizzati

// con questo metodo dico quanti figli ha l'item 
- (int)	
outlineView:			(NSOutlineView *)outlineView 
	numberOfChildrenOfItem:	(id)item
 {

	// se l'item e' nil, stiamo parlando della radice
	if (item == nil)
		// dico quindi che ci sono tanti elementi quanti presenti nel vettore
		return( [ [self startPoint] count ]);
	// se arrivo qui, mi si chiede quali figli ha un file
	// tratto subito i file normali, che non hanno figli
	if ( [item numOfFiles] == 0 )
		return ( 0 ) ;
	// se arrivo qui, ho una directory
	// se devo mostrare anche i dotfiles, conto tutto
	if ( [[ UserPrefs getPrefValue: keyShowDotFiles] boolValue] )
		return ( [item numOfFiles] ) ;
	// se arrivo qui, devo eliminare dal computo i dotFiles
	// ed alora, mi tocca esaminare tutti i file e contare quelli
	// che mi interessano
	return ( countNormalFiles( item) ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// con questo metodo dico se item e' espandibile
- (BOOL) 
outlineView:			(NSOutlineView *)outlineView 
	isItemExpandable:	(id)item 
{	
	if (item == nil)
		return ( YES ) ;
	// se non ha figli, e' un file, non si espande
	if ( [item numOfFiles] == 0 )
		return ( NO ) ;
	// se arrivo qui, l'item ha figli
	// se devo espandere anche i bundle, espando tutto
	if (  [[ UserPrefs getPrefValue:keyExpandBundle] boolValue] )
		return ( YES ) ;
	// se arrivo qui, non devo espandere i bundle
	// espando allora i non bundle
	return (! checkIfBundleDirectory([ item fileName ]) ) ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// con questo metodo dico qual e' il figlio "index-simo" di item
- (id)
outlineView:		(NSOutlineView *) outlineView 
	child:		(int) index 
	ofItem:		(id) item 
{
	NSArray * newArray ;
	NSMutableArray * tmpArray ;
	if (item == nil)
	{
		// ordino gli elementi secondo criterio
		if ( [ self orderDirection ] )
			newArray = [ [self startPoint] sortedArrayUsingFunction: ascSortFunc 
				context: [ self orderColumn]] ;
		else	newArray = [ [self startPoint] sortedArrayUsingFunction: desSortFunc 
				context: [ self orderColumn]] ;
		// trasformo il risultato in un NSMutableArray
		tmpArray = [ NSMutableArray arrayWithCapacity: [ newArray count]];
		[ tmpArray addObjectsFromArray: newArray ];
		return( [ tmpArray objectAtIndex: index ]);
	}
	// negli altri casi, recupero la lista
	[ item sortFileList: [ self orderColumn] direction: [ self orderDirection ] ];
	// se visualizzo anche i dotFiles, non c'e' problema
	if ( [[ UserPrefs getPrefValue:keyShowDotFiles] boolValue] )
		return( [item getFileAtIndex:index]);
	// altrimenti, e' un bel pasticcio, devo saltare i dotfiles
	return ( getNormalFile ( item, index ) );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// con questo metodo dico cosa deve mostrare item nella colonna indicata
- (id) 
outlineView:				(NSOutlineView *) outlineView 
	objectValueForTableColumn:	(NSTableColumn *) tableColumn 
	byItem:				(id)item 
{
	NSString * colId  ;
	
	// se l'oggetto e' vuoto, c'e' qualche problema...
	if (item == nil)
	{
		NSLog( @"Error getting data for the catalog");
		return ( @"????" ) ;
	}
	// recupero l'identificatore della colonna
	colId = [ tableColumn identifier] ;
	// se devo mostrare il nome del file, assegno l'icona
	if ( [colId isEqualToString: COLID_FILENAME] ) 
	{
		// ricavo la cella ed imposto l'immagine
		[((ImageAndTextCell*) [tableColumn dataCell]) setImage: [item fileIcon] ];
	} 
	// se devo mostrare la dimensione del file, tratto il long long a parte
	if ( [ colId isEqual: COLID_FILESIZE ] )
	{
		long long	tmp ;
		tmp = [ item fileSize ] ;
		return ( [ NSNumber numberWithLongLong: [ item fileSize ]] );	
	}
	// in tutti gli altri casi, uso il metodo valueForKey
	return ( [ item valueForKey: colId ] ) ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// la colonna non e' editabile...
- (BOOL)
outlineView:			(NSOutlineView *) outlineView 
	shouldEditTableColumn:	(NSTableColumn *) tableColumn 
	item:			(id) item 
{
    return NO;
}

// --------------------------------------------------------------------- 
// metodo che effettua una ricerca di tutti gli elementi che rispondono
// ad un dato criterio, e li infila in un vettore

- (void) 
searchFor: (id) item2search 
	column: (NSString*) colId 
	options: (int) opt 
	placeIn: (NSMutableArray*) foundItems 
{
	// piglio un elenco di tutti i volumi presenti
	NSEnumerator 	* enumerator = [[self startPoint] objectEnumerator];
	FileStruct	* item ;
		// li esamino uno ad uno
	while (item = [enumerator nextObject])
	{	
		// e faccio una ricerca in profondita' sugli elementi contenuti
		[ item searchItemFor:  item2search column: colId 
			options: opt  placeIn:  foundItems ];
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo che espande gli elementi di un outlineView fino all'elemento
// indicato

- (void) 
expandDStoItem: (FileStruct*) expItem inOutlineView: (NSOutlineView *) outView
{
	// recupero il path completo dell'elemento obiettivo
	NSString	* fPath = [expItem fileFullPath] ;
	// lo spezzo ordinamente nei vari elementi
	NSEnumerator 	* pathComps = [[fPath pathComponents] objectEnumerator];
	// piglio anche la lista dei volumi
	NSEnumerator 	* listavolumi = [[self startPoint] objectEnumerator];
	FileStruct	* item ;
	// pulisco i primi elementi del percorso completo dell'obiettivo
	NSString	* name = [pathComps nextObject] ;	// "/"
	name = [pathComps nextObject] ;			// "Volume"
	name = [pathComps nextObject] ;			// nome volume
	
	// cerco nella lista dei volumi il volume che mi serve
	while (item = [listavolumi nextObject])
	{
		// quando ho trovato il volume...
		if ( [[item fileName] isEqual:  name] )
		{
			int	thisRow = [outView rowForItem: item];
			// dico che questo item va espanso perche' fa parte della
			// catena che porta all'obiettivo
			[ outView expandItem: item ];
			// lo seleziono
			[ outView selectRow: thisRow  byExtendingSelection: FALSE ];
			// e lo porto in bella vista
			[ outView scrollRowToVisible: thisRow ];
			// poi proseguo nell'espansione
			[ item expandFStoItem: pathComps inOutlineView: outView] ;
			[ outView collapseItem: expItem ];
			return ;
		}
	}
}

@end


// --------------------------------------------------------------------- 
// funzioni ausiliari alla gestione della sorgente dati

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// funzione che dice se il file va bene o meno
BOOL
skipDotFile ( NSString * fileName )
{
	const char	* fn ;		// nome del file in C
	// trasformo il nome del file in una stringa C
	fn = [ fileName cString] ;
	// perche' poi cosi' escludo quelli che cominciano con '.'
	if (*fn == '.' )
		return ( TRUE ) ;
	// escludo anche i file che si chiamano "Icon\r"
	// non chiedete il perche' delle '\r'
	if ( [ fileName isEqual: @"Icon\r"] )
		return ( TRUE ) ;
	// ... altre condizioni di esclusione
	// - - - - - - - - - - - - - - - -
	return ( FALSE );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// conto quanti file "normali" sono presenti in una directory
// per file normali si intendono file il cui nome non comincia con '.'
// file che non si chiamano "Icon" o in qualche altro modo noto

short
countNormalFiles( FileStruct * directory )
{
	short	i, count ;

	count = 0 ;	// all'inizio, nessuno
	for ( i = 0 ; i < [directory numOfFiles] ; i++ )
	{
		// se devo saltarlo, vado avanti senza incrementare il contatore
		if ( skipDotFile( [[directory getFileAtIndex:i] fileName ] ) )
			continue ;
		// se arrivo qui, e' un file normale, incremento il contatore
		count += 1 ;
	}
	return ( count );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// per riconoscere se una directory e' un bundle, non ho altre possibilita'
// che confrontare l'estensione del file...
BOOL 
checkIfBundleDirectory( NSString * fileName )
{
	// per ricavare l'estensione, c'e' un metodo apposito
	if ( [ fileName hasSuffix: @".app"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".bundle"] )
		return ( YES );
	if ( [ fileName hasSuffix: @".framework"] )
		return ( YES );
	return ( NO );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// restituisce l'i-esimo file normale all'interno di una directory
// saltando appunto i dotFiles
FileStruct *
getNormalFile ( FileStruct * directory, short index )
{
	short	i, count ;

	count = 0 ;	// contatore corrente
	for ( i = 0 ; i < [directory numOfFiles] ; i++ )
	{
		FileStruct * currFile ;
		// recupero il file
		currFile = [directory getFileAtIndex:i] ;
		// se devo saltarlo, passo avanti
		if ( skipDotFile( [ currFile fileName ] ) )
			continue ;
		// sono arrivato all'indice richiesto ?
		if ( count == index )
			// si, restituisco l'elemento corrente
			return ( currFile );
		// se arrivo qui, non ho ancora raggiunto l'elemento
		count += 1 ;
	}
	// ovviamente, qui non dovrei mai arrivare...
	return ( nil) ;
}

// --------------------------------------------------------------------- 
