// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  lsFileInfo.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe che modella le informazioni relative ad un file

#import "LSFileInfo.h"

#import		"djZeroUtils.h"
#import		"WaitPanCtrl.h"

@implementation LSFileInfo

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// inizializzazione dell'istanza in base al nome completo
- (id) 
initWithPath: (NSString*) aFile
{
	NSImage		* tmpImg ;

	// piglio l'oggetto FileManager di default
	NSFileManager	*manager = [NSFileManager defaultManager];
	// ed in base a questo recupero le informazioni proprie del file
	NSDictionary	*fattrs	 = [manager fileAttributesAtPath: aFile traverseLink:NO];

	// inizializzo la classe madre
	[ super init ];
	// eseguo l'animazione della progress bar
	[ WaitPanCtrl animate: self ];
	// ed ora passo ad inizializzare tutte le variabili d'istanza
	// path completo e nome sono facili
	[ self setFileFullPath: aFile ];
	// qui uso un metodo standard per recuperare il nome dal path completo
	[ self setFileName: [aFile lastPathComponent]];
	// con i metodi di comodo della classe NSDictionary recupero tutte le 
	// info codificate
	[ self setModDate: [fattrs fileModificationDate] ];
	[ self setFileSize: [ fattrs fileSize ] ];
	[ self setOwnGroupName: [ fattrs fileGroupOwnerAccountName ] ];
	[ self setOwnerName:[ fattrs fileOwnerAccountName ] ];
	[ self setFilePosixPerm: [ fattrs filePosixPermissions ] ] ;
	[ self setFileType: [ fattrs fileType ] ] ;
	// questi metodi di comodo non li ho visti la prima volta che 
	// ho fatto questa classe (o forse non c'erano)
	[ self setFsFileNum: [ fattrs fileSystemFileNumber] ];
	[ self setFsNum: [ fattrs fileSystemNumber] ];
	// questi metodi proprio non sono documentati (ma ci sono nel .h)
	[ self setCreatorCode: [ fattrs fileHFSCreatorCode] ];
	[ self setTypeCode: [ fattrs fileHFSTypeCode] ];
	[ self setExtHidden: [ fattrs fileExtensionHidden] ];
	// per leggere questo dato non c'e' altra via che quella piu' lunga:
	// leggo un elemento dal dizionario tramite la chiave
	// lo trasformo nell'unita' che mi interessa (long) e poi lo assegno
	[ self setRefCount: [ [ fattrs objectForKey: NSFileReferenceCount ] longValue] ];
	// prelevo l'icona del file
	tmpImg = [ [ NSWorkspace sharedWorkspace] iconForFile: aFile ] ;
	// qui ci sono molte "rappresentazioni" dell'icona...
	reduceImageToIcon( tmpImg );
	// e poi l'assegno all'interno del file
	[ self setFileIcon: tmpImg ];
	return ( self ) ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// dealloco tutta la memoria che ho dovuto costruire 
// in pratica, le stringhe e l'icona
- (void) 
dealloc 
{
	[ fileIcon release ];
	[ fileFullPath release ];
	[ fileName release ] ;
	[ modDate release ] ;
	[ ownGroupName release ] ;
	[ ownerName release ];
	[ fileType release ] ;	
	[ super dealloc ] ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo per il salvataggio/ripristino dell'istanza da file
- (void)
encodeWithCoder:(NSCoder *)encoder 
{
	// non devo chiamare super perche' sono diretto discendente di NSObject
	// gli oggetti li scrivo col metodo appropriato
	[ encoder encodeObject: [ self fileFullPath ] ] ;
	[ encoder encodeObject: [ self fileName ] ] ;
	[ encoder encodeObject: [ self modDate ] ] ;
	[ encoder encodeObject: [ self ownGroupName ] ] ;
	[ encoder encodeObject: [ self ownerName ] ] ;
	[ encoder encodeObject: [ self fileType ] ] ;
	[ encoder encodeObject: [ self fileIcon ] ] ;
	// tutti gli altri con i metodi specifici dei tipi C
	[ encoder encodeValueOfObjCType: @encode(long long) at: & fileSize];
	[ encoder encodeValueOfObjCType: @encode(unsigned long) at: & filePosixPerm];
	[ encoder encodeValueOfObjCType: @encode(OSType) at: & creatorCode];
	[ encoder encodeValueOfObjCType: @encode(OSType) at: & typeCode];
	[ encoder encodeValueOfObjCType: @encode(long) at: & refCount];
	[ encoder encodeValueOfObjCType: @encode(unsigned long) at: & fsFileNum];
	[ encoder encodeValueOfObjCType: @encode(unsigned long) at: & fsNum];
	[ encoder encodeValueOfObjCType: @encode(BOOL) at: & extHidden];
}

- (id)initWithCoder:(NSCoder *)decoder 
{
	// e' fondamentale l'ordine, che deve essere lo stesso di encodeWithCoder
	// non devo chiamare super perche' sono diretto discendente di NSObject
	[ self setFileFullPath: [ decoder decodeObject ] ];
	[ self setFileName: [ decoder decodeObject ] ];
	[ self setModDate: [ decoder decodeObject ] ];
	[ self setOwnGroupName: [ decoder decodeObject ] ];
	[ self setOwnerName: [ decoder decodeObject ] ];
	[ self setFileType: [ decoder decodeObject ] ];
	[ self setFileIcon: [ decoder decodeObject ] ];
	[ decoder decodeValueOfObjCType: @encode(long long) at: & fileSize];
	[ decoder decodeValueOfObjCType: @encode(unsigned long) at: & filePosixPerm];
	[ decoder decodeValueOfObjCType: @encode(OSType) at: & creatorCode];
	[ decoder decodeValueOfObjCType: @encode(OSType) at: & typeCode];
	[ decoder decodeValueOfObjCType: @encode(long) at: & refCount];
	[ decoder decodeValueOfObjCType: @encode(unsigned long) at: & fsFileNum];
	[ decoder decodeValueOfObjCType: @encode(unsigned long) at: & fsNum];
	[ decoder decodeValueOfObjCType: @encode(BOOL) at: & extHidden];
	return self ;
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodi accessor per tutte le variabili d'istanza

- (NSString*)fileFullPath
	{ return fileFullPath; }
- (void)setFileFullPath:(NSString*)newFileFullPath
	{ OBJ_ACC_SET( fileFullPath, newFileFullPath ) ; }

- (NSString*)fileName
	{ return fileName; }
- (void)setFileName:(NSString*)newFileName
	{ OBJ_ACC_SET( fileName, newFileName ) ; }

- (NSDate*)modDate
	{ return modDate; }
- (void)setModDate:(NSDate*)newModDate
	{ OBJ_ACC_SET( modDate, newModDate ) ; }

- (unsigned long long)fileSize
	{ return fileSize; }
- (void)setFileSize:(unsigned long long)newFileSize
	{ fileSize = newFileSize; }

- (NSString*)ownGroupName
	{ return ownGroupName; }
- (void)setOwnGroupName:(NSString*)newFgoan
	{ OBJ_ACC_SET( ownGroupName, newFgoan ) ; }

- (NSString*)ownerName
	{ return ownerName; }
- (void)setOwnerName:(NSString*)newFoan
	{ OBJ_ACC_SET( ownerName, newFoan ) ; }

- (unsigned long)filePosixPerm
	{ return filePosixPerm; }
- (void)setFilePosixPerm:(unsigned long)newFilePosixPerm
	{  filePosixPerm = newFilePosixPerm; }

- (NSString*)fileType
	{ return fileType; }
- (void)setFileType:(NSString*)newFileType
	{ OBJ_ACC_SET( fileType, newFileType ) ; }

- (OSType)creatorCode
	{  return creatorCode; }
- (void)setCreatorCode:(OSType)newCreatorCode
	{  creatorCode = newCreatorCode; }

- (OSType)typeCode
	{ return typeCode; }
- (void)setTypeCode:(OSType)newTypeCode
	{ typeCode = newTypeCode; }

- (long)refCount 
	{ return refCount; }
- (void)setRefCount: (long) newrefCount
	{ refCount = newrefCount; }

- (unsigned long)fsFileNum
	{ return fsFileNum; }
- (void)setFsFileNum: (unsigned long) newfsFileNum
	{ fsFileNum = newfsFileNum; }

- (unsigned long)fsNum
	{ return fsNum; }
- (void)setFsNum: (unsigned long) newfsNum
	{ fsNum = newfsNum; }

- (BOOL)extHidden
	{ return extHidden; }
- (void)setExtHidden: (BOOL) newExtHidden
	{ extHidden = newExtHidden; }
	
- (NSImage *)fileIcon
	{ return fileIcon; }
- (void)setFileIcon: (NSImage *) newicon
	{ OBJ_ACC_SET(fileIcon, newicon); }

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
@end
