// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  UserPrefs.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe per la gestione delle preference

#import "UserPrefs.h"

#import		"djZeroUtils.h"

// funzini ausiliarie per la lettura/scrittura delle preferenze
void	readBoolDefault ( NSMutableDictionary * locDict, NSUserDefaults * locDef, id key );
void	writeBoolDefault( NSMutableDictionary * locDict, NSUserDefaults * locDef, id key ) ;

@implementation UserPrefs

// istanza unica delle preferenze
static UserPrefs		* locSharedPref  = nil ;
// dizionario delle preferenze coi valori stabiliti dal programma
static NSMutableDictionary 	* defCodeValues = nil ;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo disponibile a tutti comodamente per leggere un valore di default
+ ( id ) getPrefValue: (id) key ;
{
    return [[locSharedPref defCurrValues] objectForKey:key];
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// per avere l'intero dizionario delle preferenze
+ ( NSMutableDictionary*) getPrefs
	{ return [ locSharedPref defCurrValues] ; }
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// costruizione della istanza
- ( id )
init
{
	// mi preoccupo che non siano create piu' istanze delle preferenze
	if (  locSharedPref )
		return ( locSharedPref );
	// se arrivo qui devo costruire l'istanza condivisa
	self = [ super init ] ;
	// costruisco il dizionario delle preferenze
	defCodeValues = [ NSMutableDictionary dictionaryWithContentsOfFile: 
		[[NSBundle mainBundle] pathForResource: @"defCodeValues" ofType: @"dict"]];
	[ defCodeValues retain ];
	// l' istanza condivisa e' proprio questa
	locSharedPref = self ;
	// carico le preferenze dal file
	[ UserPrefs loadPrefsFromDefault ];
	return (self );	// restituisco l'istanza
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodi accessor
- ( NSMutableDictionary*) defCurrValues			{ return defCurrValues ; }
- (void) setDefCurrValues: ( NSMutableDictionary*) new	OBJ_ACC_SET( defCurrValues, new )
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
+ (void) 
loadPrefsFromDefault 
{
	// pesco il file dei defaults e costruisco un dizionario
	NSUserDefaults 	*defaults = [NSUserDefaults standardUserDefaults];
	NSMutableDictionary *dict = [NSMutableDictionary dictionaryWithCapacity:10];
	// leggo ordinatamente i vari elementi
	readBoolDefault( dict, defaults, keyExpandBundle );
	readBoolDefault( dict, defaults, keyShowDotFiles );
	readBoolDefault( dict, defaults, keyColFullPath );
	readBoolDefault( dict, defaults, keyColModDate );
	readBoolDefault( dict, defaults, keyColFileSize );
	readBoolDefault( dict, defaults, keyColGroupName );
	readBoolDefault( dict, defaults, keyColOwnerName );
	readBoolDefault( dict, defaults, keyColPosixPerm );
	readBoolDefault( dict, defaults, keyColFileType );
	readBoolDefault( dict, defaults, keyColCreator );
	readBoolDefault( dict, defaults, keyColOSType );
	readBoolDefault( dict, defaults, keyColFsFileNum );
	readBoolDefault( dict, defaults, keyColFsNum );
	// assegno il nuovo dizionario
	[ locSharedPref setDefCurrValues: dict ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
+ (void)
savePrefsToDefault: ( NSMutableDictionary*) locDict 
{
	// recupero il file dei defaults
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	// inserisco ordinatamente i vari elementi
	writeBoolDefault( locDict, defaults, keyExpandBundle );
	writeBoolDefault( locDict, defaults, keyShowDotFiles );
	writeBoolDefault( locDict, defaults, keyExpandBundle );
	writeBoolDefault( locDict, defaults, keyShowDotFiles );
	writeBoolDefault( locDict, defaults, keyColFullPath );
	writeBoolDefault( locDict, defaults, keyColModDate );
	writeBoolDefault( locDict, defaults, keyColFileSize );
	writeBoolDefault( locDict, defaults, keyColGroupName );
	writeBoolDefault( locDict, defaults, keyColOwnerName );
	writeBoolDefault( locDict, defaults, keyColPosixPerm );
	writeBoolDefault( locDict, defaults, keyColFileType );
	writeBoolDefault( locDict, defaults, keyColCreator );
	writeBoolDefault( locDict, defaults, keyColOSType );
	writeBoolDefault( locDict, defaults, keyColFsFileNum );
	writeBoolDefault( locDict, defaults, keyColFsNum );
	// dico a tutti che le pref sono state aggiornate
	[[NSNotificationCenter defaultCenter]
		postNotificationName: PREF_UPDATE_NOTIFICATION object:self];
}
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 

@end

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// funzione per leggere un valore booleano come default
void
readBoolDefault ( NSMutableDictionary * locDict, NSUserDefaults * locDef, id key )
{
	// vedo se nei defaults c'e' un valore per la chiave
	if ( [ locDef objectForKey: key] )
	{
		// recupero dal dizionario il booleano, poi lo converto in un NSNumber 
		// per reinfilarlo nel dizionario
		[locDict setObject: [NSNumber numberWithBool:[locDef boolForKey:key]] forKey:key] ;
	}
	else
	{
		// recupero dal dizionario def-code il valore e lo metto del dzionario
	 	[locDict setObject: [defCodeValues objectForKey:key] forKey:key] ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// funzione per scrivere un valore booleano come default
void
writeBoolDefault( NSMutableDictionary * locDict, NSUserDefaults * locDef, id key )
{
	// se il valore def-code e' lo stesso del valore def-file
	if ( [ [defCodeValues objectForKey: key] isEqual: [locDict objectForKey: key] ] )
	{
		// tolgo addirittura il valore dal file; 
		// in questo modo lo tengo pulito e conciso
		[locDef removeObjectForKey: key]; 
	}
	else 
	{
		// altrimenti, lo scrivo nel file
		[locDef setBool:[[locDict objectForKey: key] boolValue] forKey: key];
	}
}
