// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  FindWinCtrl.m
//  Created by djzero00 -- Copyright (c) 2002
//
// Finestra per effettuare ricerche in un catalogo

#import 	"FindWinCtrl.h"

#import		"FileStruct.h"
#import		"LSDataSource.h"
#import 	"WaitPanCtrl.h"
#import		"djZeroUtils.h"

@implementation FindWinCtrl

// --------------------------------------------------------------------- 
// i soliti metodi classici per condividere una unica finestra, 
// inizializzarla, ucciderla e caricarla

+ (id) sharedFind
{
	static FindWinCtrl * _sharedFind = nil ;

	if ( ! _sharedFind )
		_sharedFind = [ [FindWinCtrl alloc] init ];
	return ( _sharedFind );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (id ) 
init
{
	self = [ self initWithWindowNibName: @"FindWin" ];
	[self setFoundItems: [ NSMutableArray array ]];	
	return self ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void) 
dealloc 
{
	[ foundItems release ] ;	
	[ super dealloc ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
- (void)
windowDidLoad 
{
	// un doppio clic su di una riga mostra l'elemento selezionato
	// (risultato di una ricerca) all'interno del documento che lo contiene
	[ resultFld setDoubleAction: @selector( goToItem:) ];
}

// --------------------------------------------------------------------- 
// metodi per fornire i dati alla tabella

- (int)
numberOfRowsInTableView:	(NSTableView *)tableView
{
	return [foundItems count];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// l'elemento da visualizzare

- (id)
tableView:			(NSTableView *)tableView 
    objectValueForTableColumn:	(NSTableColumn *)tableColumn 
    row: 			(int)row
{
	LSFileInfo	* item = [foundItems objectAtIndex: row] ;
	NSString	* colId = [ tableColumn identifier] ;

	// ho al momento due colonne: il nome o l'intero percorso
	if ( [ colId isEqual: COLID_FILENAME ] )
		return ( [ item fileName] );			
	return ( [ item fileFullPath] );	
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo per indicare se la cella e' editabile o meno
- (BOOL)
tableView:			(NSTableView *)aTableView 
	shouldEditTableColumn:	(NSTableColumn *)aTableColumn 
	row:			(int)rowIndex
{
	return NO ;
}

// --------------------------------------------------------------------- 
// cambia il menu delle opzioni a seconda del campo scelto

- (IBAction)
changeCfrMenu:(id)sender
{
	// uso il tag del menu che ho accortamente predisposto in IB
	switch ( [[ whereOpt selectedItem] tag ] ) {
	// questi sono tutte strighe
	case MENUTAG_FILENAME:		case MENUTAG_FULLPATH:	
	case MENUTAG_GROUPNAME:		case MENUTAG_OWNERNAME:	
	case MENUTAG_FILETYPE:
		[ howOpt setMenu: stringMenu ];
		break ;
	// questi sono numeri, oppure date
	case MENUTAG_MODDATE: 		case MENUTAG_FILESIZE: 
	case MENUTAG_POSIXPERM: 	case MENUTAG_OSCREATOR: 
	case MENUTAG_OSTYPE: 		case MENUTAG_FSFILENUM: 
	case MENUTAG_FSNUM:
		[ howOpt setMenu: numberMenu ];
		break ;
	}
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo che effettua la ricerca all'interno della finestra di catalogo
// principale (quella che sta davanti alle altre...)

- (IBAction)
startSearch:(id)sender
{
	// inizializzo questo array per selezionare poi la colonna
	NSArray	* colIdArr = [NSArray arrayWithObjects:
		COLID_FILENAME, COLID_FULLPATH, COLID_MODDATE, 
		COLID_FILESIZE, COLID_GROUPNAME, COLID_OWNERNAME,
		COLID_POSIXPERM, COLID_FILETYPE, COLID_OSCREATOR,
		COLID_OSTYPE, COLID_FSFILENUM, COLID_FSNUM,
		nil];
	// recupero cosa c'e' da cercare
	NSString	* txt2search = [ FindText stringValue];
	// recupero le opzioni di ricerca
	int		how = [[ howOpt selectedItem] tag ] ;
	// ed il campo dove cercare
	int		where = [[ whereOpt selectedItem] tag ] ;
	// mi faccio la finestra di attesa
	WaitPanCtrl	 *tmpCVCtrl = [[ WaitPanCtrl alloc] init ] ;
	// faccio comparire la sheet che dice di attendere
	[ NSApp beginSheet: [ tmpCVCtrl window ]
		modalForWindow: [ NSApp mainWindow ]  
		modalDelegate: self
		didEndSelector: nil  
		contextInfo: nil
	];
	// imposto il contenuto della shett stessa
	[ tmpCVCtrl setMainText: @"Searching" ];
	[ tmpCVCtrl animation: self ];
	// recupero il catalogo dove effettuare la ricerca
	// piglio tutti i documenti dell'applicazione, e scelgo quello che sta
	// davanti a tutti gli altri... non che questo meccanismo mi piaccia...
	[ self setCurrCatalog: (CatalogDoc*)[[ NSApp orderedDocuments] objectAtIndex: 0] ];
	
	// pulisco i risultati della ricerca precedente
	[self setFoundItems: [ NSMutableArray array ]];	
	// e li tolgo dalla finestra di ricerca
	[ resultFld reloadData ];
	// adesso eseguo la ricerca, in maniera brutale, sulla sorgente dati
	[ [ [self currCatalog] dataSource ] searchFor: txt2search 
		column: [ colIdArr objectAtIndex: where] 
		options: how placeIn: foundItems];
	// forzo il rinfresco della finestra dei risultati
	[ resultFld reloadData ];
	// nascondo e chiudo la sheet
	[ [ tmpCVCtrl window ] setIsVisible: FALSE ];
	[ NSApp endSheet: [ tmpCVCtrl window ] ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// metodo chiamato da un doppio clic sulla tabella con i risultati della ricerca

- (IBAction)
goToItem:(id)sender
{
	FileStruct	* item ;
	// verifico che ci sia una riga selezionata
	if ( [ sender selectedRow ] == -1 ) return ;
	// recupero l'elemento selezionato
	item = [ foundItems objectAtIndex: [ sender selectedRow ]];
	// controllo che il documento cui si riferisce la ricerca sia ancora presente
	if ( ! [ [ NSApp orderedDocuments] containsObject: [self currCatalog] ] )
	{
		// il documento non c'e' piu', informo l'utente con un alert
		NSBeginAlertSheet(@"Missing Catalog", @"Ok, ok", nil, nil, 
			[self window], nil, nil, nil, nil, 
			@"The Catalog Document is no longer present") ;
		// non ho altro da fare...
		return ;
	}
	// qui abbiamo il catalogo, ed abbiamo l'item; lo cerchiamo e lo apriamo
	[[ [self currCatalog] dataSource ] expandDStoItem: item 
		inOutlineView: [ [self currCatalog] outlineView ] ];
	// forzo il redisplay della finestra del catalogo
	[ [ [self currCatalog] outlineView ] reloadData ];
	// metto la finestra del catalogo davanti alle altre
	[ [self currCatalog] showWindows ] ;
}

// --------------------------------------------------------------------- 
// i soliti metodi accessor

- (NSMutableArray *) foundItems			{ return foundItems ; }
- (void) setFoundItems: (NSMutableArray*) data	OBJ_ACC_SET( foundItems, data )

- (CatalogDoc *) currCatalog			{ return currCatalog ; }
- (void) setCurrCatalog: (CatalogDoc*) data	OBJ_ACC_SET( currCatalog, data )

@end
