// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
//  InfoWinCtrl.m
//  Created by djzero00 - - Copyright (c) 2002
//
// realizzazione della classe per la gestione delle info sul file

#import 	"InfoWinCtrl.h"

#import		"LSFormatter.h"
#import 	"FileStruct.h"
#import 	"CatalogDoc.h"

@implementation InfoWinCtrl
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// costruisto l'istanza condivisa
+ ( id ) 
sharedInfoWinCtrl 
{
	static	InfoWinCtrl	* locSharedWinCtrl  = nil ;
	
	if ( ! locSharedWinCtrl )
	{
		locSharedWinCtrl = [[ InfoWinCtrl allocWithZone: [self zone]] init ] ;
	}
	return ( locSharedWinCtrl );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// inizializzo la finestra caricandola dal file nib
- (id )
init 
{
	self = [ self initWithWindowNibName: @"infoPanel" ];
	return ( self );
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// preparo la finestra la suo lavoro
- (void)
windowDidLoad 
{
	// costruisco i formattatori che mi servono
	TOS9TCForm 	*myDF1 = [[[ TOS9TCForm alloc   ] init ] autorelease ];
	FileSizeForm 	*myDF2 = [[[ FileSizeForm alloc ] init ] autorelease ];
	FilePosixPerm 	*myDF3 = [[[ FilePosixPerm alloc ] init ] autorelease ];
	[ super windowDidLoad ] ;
	// installo i formattatori
	[ osCreator setFormatter:  myDF1 ];
	[ osType setFormatter:  myDF1 ];
	[ fileSize setFormatter:  myDF2 ];
	[ posixPerm setFormatter:  myDF3 ];
	// dico che il winctrl osserva le notifiche riguardanti
	// 1. il cambiamento di selezione di una outlineView
	// eseguendo il metodo selectionChanged:
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( selectionChanged: )
		name: NSOutlineViewSelectionDidChangeNotification object: nil ] ;
	// 2. il cambiamento di finestra
	// eseguendo il metodo mainWindowChanged:
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( mainWindowChanged: )
		name: NSWindowDidBecomeMainNotification object: nil ] ;
	// 3. la chiusura di finestra
	// eseguendo il metodo mainWindowResigned:
	[[ NSNotificationCenter defaultCenter] addObserver: self
		selector: @selector( mainWindowResigned: )
		name: NSWindowDidResignMainNotification object: nil ] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// nella deallocazione, rimuovo l'osservatore
- (void)
dealloc
{
	[[ NSNotificationCenter defaultCenter] removeObserver: self ];
	[ super dealloc ];
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// aggiorno la finestra quando cambia la sezione o la finestra
// outView e' la nuova outlineView che va utilizzata
- (void)
updateInfo: (NSOutlineView *) outView 
{
	long		row ;
	FileStruct	* locItem ;
	// vedo se ci sono selezioni in corso
	if ( outView != nil )
		row = [ outView selectedRow] ;
	else 	row = -1 ;
	// se non ce ne sono, faccio nulla
	if ( row == -1 )
	{
		// ripulisco la finestra
		[ fileType setStringValue: @""];
		[ groupName setStringValue: @""];
		[ ownerName setStringValue: @""];
		[ fullPath setStringValue: @""];
		[ fsFileNum setStringValue: @"" ];
		[ fsNum setStringValue: @"" ] ;
		[ fileSize setObjectValue: nil ];	
		[ osCreator setObjectValue:  nil ] ;
		[ osType setObjectValue: nil ];
		[ posixPerm setObjectValue:  nil ];
		[ modDate setObjectValue: nil ];
		[ fileIcon setObjectValue : nil ];
		[[ self window ] setTitle: @"No Info" ] ;
		return ;
	}
	// altrimenti, uso l'elemento selezionato
	locItem = [ outView itemAtRow: row ] ;
	// se l'oggetto e' reale, aggiorno il contenuto
    	[ fileType setStringValue: [ locItem fileType]];
    	[ groupName setStringValue: [ locItem ownGroupName]];
    	[ ownerName setStringValue: [ locItem ownerName]];
    	[ fullPath setStringValue: [ locItem fileFullPath]];
	// piglio l'intero e lo traformo in stringa
	[ fsFileNum setStringValue: [ NSString stringWithFormat: @"%d", [ locItem fsFileNum]] ];
	[ fsNum setStringValue: [ NSString stringWithFormat: @"%d", [ locItem fsNum]]] ;
	// questi hanno un formattatore appiccicato, gli passo direttamente l'intero
	[ fileSize setObjectValue: [NSNumber numberWithLongLong: [ locItem fileSize] ]];	
	[ osCreator setIntValue:  [ locItem creatorCode]] ;
	[ osType setIntValue: [ locItem typeCode] ];
	[ posixPerm setIntValue:  [ locItem filePosixPerm] ];
	// qui, con il formattatore di data, gli passo direttamente l'oggetto NSDate
    	[ modDate setObjectValue: [ locItem modDate]];
	// recupero l'icona del file
	[ fileIcon setObjectValue : [locItem fileIcon] ];
	// infine, il nome del file e' il titolo della finestra
	[[ self window ] setTitle: [ locItem fileName ]] ;
}

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
// questo metodo e' eseguito quando cambia la selezione all'interno di una
// outlineView
- (void )
selectionChanged: ( NSNotification *) notification
{
	[ self updateInfo: [ notification object ] ];
}

// questo metodo e' eseguito quando una finestra e' posta davanti a tutti
- (void )
mainWindowChanged: ( NSNotification *) notification
{
	[ self updateInfo: [[[[ notification object] windowController] document] outlineView] ];
}

// questo metodo e' eseguito quando una finestra e' posta dietro oppure e' chiusa
- (void)
mainWindowResigned:(NSNotification *) notification 
{
	[self updateInfo: nil];
}
@end
